@@ -8,20 +8,22 @@ use warnings;
 require 5.00404;  ##Yes, this is underhanded, but makes support for me easier
 		  ##Not only that, but it's the latest "safe" version of
 		  ##Perl5.  01-03 weren't bug free.
-use vars (qw($VERSION $Tolerance));
+use vars (qw($VERSION $Tolerance $Min_samples_number));
 
-$VERSION = '3.0202';
+$VERSION = '3.0604';
 
 $Tolerance = 0.0;
+$Min_samples_number = 4;
 
 package Statistics::Descriptive::Sparse;
 
 use vars qw($VERSION);
 
-$VERSION = '3.0202';
+$VERSION = '3.0604';
 
 use vars qw(%fields);
 use Carp;
+use Statistics::Descriptive::Smoother;
 
 sub _make_accessors
 {
@@ -76,15 +78,15 @@ sub _make_private_accessors
 ##Define the fields to be used as methods
 %fields = (
   count			=> 0,
-  mean			=> 0,
-  sum			=> 0,
-  sumsq			=> 0,
+  mean			=> undef,
+  sum			=> undef,
+  sumsq			=> undef,
   min			=> undef,
   max			=> undef,
   mindex		=> undef,
   maxdex		=> undef,
   sample_range		=> undef,
-  variance => undef,
+  variance              => undef,
   );
 
 __PACKAGE__->_make_accessors( [ grep { $_ ne "variance" } keys(%fields) ] );
@@ -176,7 +178,7 @@ sub add_data {
   $self->count($count);
   ##indicator the value is not cached.  Variance isn't commonly enough
   ##used to recompute every single data add.
-  $self->_variance(undef());
+  $self->_variance(undef);
   return 1;
 }
 
@@ -189,6 +191,9 @@ sub standard_deviation {
 ##Return variance; if needed, compute and cache it.
 sub variance {
   my $self = shift;  ##Myself
+
+  return undef if (!$self->count());
+  
   my $div = @_ ? 0 : 1;
   my $count = $self->count();
   if ($count < 1 + $div) {
@@ -234,11 +239,11 @@ package Statistics::Descriptive::Full;
 
 use vars qw($VERSION);
 
-$VERSION = '3.0202';
+$VERSION = '3.0604';
 
 use Carp;
-
 use POSIX ();
+use Statistics::Descriptive::Smoother;
 
 use vars qw(@ISA $a $b %fields);
 
@@ -248,12 +253,13 @@ use vars qw(@ISA $a $b %fields);
 %fields = (
   _permitted => undef,  ##Place holder for the inherited key hash
   data       => undef,  ##Our data
+  samples    => undef,  ##Number of samples for each value of the data set
   presorted  => undef,  ##Flag to indicate the data is already sorted
   _reserved  => undef,  ##Place holder for this lookup hash
 );
 
 __PACKAGE__->_make_private_accessors(
-    [qw(data frequency geometric_mean harmonic_mean 
+    [qw(data samples frequency geometric_mean harmonic_mean
         least_squares_fit median mode
         skewness kurtosis
        )
@@ -267,6 +273,7 @@ sub _clear_fields
 
     # Empty array ref for holding data later!
     $self->_data([]);
+    $self->_samples([]);
     $self->_reserved(\%fields);
     $self->presorted(0);
     $self->_trimmed_mean_cache(+{});
@@ -346,11 +353,99 @@ sub add_data {
   return 1;
 }
 
+sub add_data_with_samples {
+    my ($self,$aref_values) = @_;
+
+    return 1 if (!@{ $aref_values });
+
+    my $aref_data = [map { keys %$_ } @{ $aref_values }];
+    my $aref_samples = [map { values %$_ } @{ $aref_values }];
+
+    $self->add_data($aref_data);
+    push @{ $self->_samples() }, @{ $aref_samples };
+
+    return 1;
+}
+
+
 sub get_data {
   my $self = shift;
   return @{ $self->_data() };
 }
 
+sub get_data_without_outliers {
+    my $self = shift;
+
+    if ($self->count() < $Statistics::Descriptive::Min_samples_number) {
+        carp("Need at least $Statistics::Descriptive::Min_samples_number samples\n");
+        return;
+    }
+
+    if (!defined $self->{_outlier_filter}) {
+        carp("Outliers filter not defined\n");
+        return;
+    }
+
+    my $outlier_candidate_index = $self->_outlier_candidate_index;
+    my $possible_outlier = ($self->_data())->[$outlier_candidate_index];
+    my $is_outlier = $self->{_outlier_filter}->($self, $possible_outlier);
+
+    return $self->get_data unless $is_outlier;
+    # Removing the outlier from the dataset
+    my @good_indexes = grep { $_ != $outlier_candidate_index } (0 .. $self->count() - 1);
+
+    my @data = $self->get_data;
+    my @filtered_data = @data[@good_indexes];
+    return @filtered_data;
+}
+
+sub set_outlier_filter {
+    my ($self, $code_ref) = @_;
+
+    if (!$code_ref || ref($code_ref) ne "CODE") {
+        carp("Need to pass a code reference");
+        return;
+    }
+
+    $self->{_outlier_filter} = $code_ref;
+    return 1;
+}
+
+sub _outlier_candidate_index {
+    my $self = shift;
+
+    my $mean = $self->mean();
+    my $outlier_candidate_index = 0;
+    my $max_std_deviation = abs(($self->_data())->[0] - $mean);
+    foreach my $idx (1 .. ($self->count() - 1) ) {
+        my $curr_value = ($self->_data())->[$idx];
+        if ($max_std_deviation  <  abs($curr_value - $mean) ) {
+            $outlier_candidate_index = $idx;
+            $max_std_deviation = abs($curr_value - $mean);
+        }
+    }
+    return $outlier_candidate_index;
+}
+
+sub set_smoother {
+    my ($self, $args) = @_;
+
+    $args->{data}    = $self->_data();
+    $args->{samples} = $self->_samples();
+
+    $self->{_smoother} = Statistics::Descriptive::Smoother->instantiate($args);
+}
+
+sub get_smoothed_data {
+    my ($self, $args) = @_;
+
+    if (!defined $self->{_smoother}) {
+        carp("Smoother object not defined\n");
+        return;
+    }
+    $self->{_smoother}->get_smoothed_data();
+}
+
 sub sort_data {
   my $self = shift;
 
@@ -368,22 +463,26 @@ sub sort_data {
 }
 
 sub percentile {
-  my $self = shift;
-  my $percentile = shift || 0;
-  ##Since we're returning a single value there's no real need
-  ##to cache this.
+    my $self = shift;
+    my $percentile = shift || 0;
+    ##Since we're returning a single value there's no real need
+    ##to cache this.
 
-  ##If the requested percentile is less than the "percentile bin
-  ##size" then return undef.  Check description of RFC 2330 in the
-  ##POD below.
-  my $count = $self->count();
-  return undef if $percentile < 100 / $count;
+    ##If the requested percentile is less than the "percentile bin
+    ##size" then return undef.  Check description of RFC 2330 in the
+    ##POD below.
+    my $count = $self->count();
 
-  $self->sort_data();
-  my $num = $count*$percentile/100;
-  my $index = &POSIX::ceil($num) - 1;
-  my $val = $self->_data->[$index];
-  return wantarray
+    if ((! $count) || ($percentile < 100 / $count))
+    {
+        return;  #  allow for both scalar and list context
+    }
+
+    $self->sort_data();
+    my $num = $count*$percentile/100;
+    my $index = &POSIX::ceil($num) - 1;
+    my $val = $self->_data->[$index];
+    return wantarray
     ? ($val, $index)
     : $val
     ;
@@ -411,6 +510,8 @@ sub _calc_new_median
 sub median {
     my $self = shift;
 
+    return undef if !$self->count;    
+    
     ##Cached?
     if (! defined($self->_median()))
     {
@@ -428,6 +529,9 @@ sub quantile {
        return;
     }
     
+    #  check data count after the args are checked - should help debugging
+    return undef if !$self->count;  
+    
     $self->sort_data();
 
     return $self->_data->[0] if ( $QuantileNumber == 0 );
@@ -490,6 +594,9 @@ sub trimmed_mean
         ($lower,$upper) = ($_[0],$_[1]);
     }
 
+    #  check data count after the args
+    return undef if !$self->count;    
+
     ##Cache
     my $thistm = join ':',$lower,$upper;
     my $cache = $self->_trimmed_mean_cache();
@@ -581,6 +688,8 @@ sub mode
 
 sub geometric_mean {
     my $self = shift;
+    
+    return undef if !$self->count;
 
     if (!defined($self->_geometric_mean()))
     {
@@ -942,10 +1051,90 @@ I<Note:  Calling add_data with an empty array will delete all of your
 Full method cached values!  Cached values for the sparse methods are
 not changed>
 
+=item $stat->add_data_with_samples([{1 => 10}, {2 => 20}, {3 => 30},]);
+
+Add data to the statistics variable and set the number of samples each value has been
+built with. The data is the key of each element of the input array ref, while
+the value is the number of samples: [{data1 => smaples1}, {data2 => samples2}, ...]
+
 =item $stat->get_data();
 
 Returns a copy of the data array.
 
+=item $stat->get_data_without_outliers();
+
+Returns a copy of the data array without outliers. The number minimum of
+samples to apply the outlier filtering is C<$Statistics::Descriptive::Min_samples_number>,
+4 by default.
+
+A function to detect outliers need to be defined (see C<set_outlier_filter>),
+otherwise the function will return an undef value.
+
+The filtering will act only on the most extreme value of the data set
+(i.e.: value with the highest absolute standard deviation from the mean).
+
+If there is the need to remove more than one outlier, the filtering
+need to be re-run for the next most extreme value with the initial outlier removed.
+
+This is not always needed since the test (for example Grubb's test) usually can only detect
+the most exreme value. If there is more than one extreme case in a set,
+then the standard deviation will be high enough to make neither case an outlier.
+
+=item $stat->set_outlier_filter($code_ref);
+
+Set the function to filter out the outlier.
+
+C<$code_ref> is the reference to the subroutine implemeting the filtering function.
+
+Returns C<undef> for invalid values of C<$code_ref> (i.e.: not defined or not a
+code reference), C<1> otherwise.
+
+=over 4
+
+=item
+
+Example #1: Undefined code reference
+
+    my $stat = Statistics::Descriptive::Full->new();
+    $stat->add_data(1, 2, 3, 4, 5);
+
+    print $stat->set_outlier_filter(); # => undef
+
+=item
+
+Example #2: Valid code reference
+
+    sub outlier_filter { return $_[1] > 1; }
+
+    my $stat = Statistics::Descriptive::Full->new();
+    $stat->add_data( 1, 1, 1, 100, 1, );
+  
+    print $stat->set_outlier_filter( \&outlier_filter ); # => 1
+    my @filtered_data = $stat->get_data_without_outliers();
+    # @filtered_data is (1, 1, 1, 1)
+
+In this example the series is really simple and the outlier filter function as well.
+For more complex series the outlier filter function might be more complex
+(see Grubbs' test for outliers).
+
+The outlier filter function will receive as first parameter the Statistics::Descriptive::Full object,
+as second the value of the candidate outlier. Having the object in the function
+might be useful for complex filters where statistics property are needed (again see Grubbs' test for outlier).
+
+=back
+
+=item $stat->set_smoother({ method => 'exponential', coeff => 0, });
+
+Set the method used to smooth the data and the smoothing coefficient.
+See C<Statistics::Smoother> for more details.
+
+=item $stat->get_smoothed_data();
+
+Returns a copy of the smoothed data array.
+
+The smoothing method and coefficient need to be defined (see C<set_smoother>),
+otherwise the function will return an undef value.
+
 =item $stat->sort_data();
 
 Sort the stored data and update the mindex and maxdex methods.  This
@@ -1221,6 +1410,10 @@ Colin Kuskie
 My email address can be found at http://www.perl.com under Who's Who
 or at: http://search.cpan.org/author/COLINK/.
 
+=head1 CONTRIBUTORS
+
+Fabio Ponciroli & Adzuna Ltd. team (outliers handling)
+
 =head1 REFERENCES
 
 RFC2330, Framework for IP Performance Metrics
